#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Monday, October 12 2020 from deaths_global2.py
for graphing deaths and positive cases for US counties

@author: davidworrall
version 0.01: Fri Jun 19 15:19:29 2020. Set up intitial cv read
version 0.02: 2020 31 Define class structure for a country
version 0.03: 2020 08 06 Add SMA
version 0.0v 0.0 write both 
    - county Data deaths          # write the incremental daily death Nrs
    - county Data Cases           # write the incremental daily cases Nrs

version 0.1 2020-10-12 finished US and counties
version 0.2 2020 10 30 begain use states from a different file: us-states.csv (also from NYTimes)

FIPS = Federal Information Processing Standard FIP codes are five-digit codes 
that are assigned to each U.S. county. See:
https://medium.com/plotly/what-is-a-fips-code-county-level-charts-in-python-4eff383a4cf6
also note: http://ghdx.healthdata.org/


"""
# ====================IMPORT STATE COUNTY DATA AND STORE IN A DICT ===========#
# Some columnanmes to enhance readability
colLabelRow  = 0
stateCol     = 0
countyCol    = 1
LatCol       = 2
LongCol      = 3
dateBeginCol = 4

import csv
directory="/Users/davidworrall/Documents/RESEARCH/COVID-19/dataSets/deaths_USstates/"
directory="/volumes/DRWORANGE32/COVID-19/dataSets/"

imageDirectory = "/volumes/DRWORANGE32/COVID-19/graphImages/cases_USstates/"

countyDeathFilename     = "countyDeathsUS.csv"
countyCasesFilename     = "countyCasesUS.csv"
populationDataFilename  = "countryPopulationData.csv"                                # "for the population and geographic data
stateDataFilemane       = "us-statwa.csv"


#with open('/Users/davidworrall/Documents/CCC/TEACHING/TeachingResources_DSon/LiteracyRatesGlobalLongRun.csv', newline='') as csvfile:
# =========== PULL THE New York RTimes DB  FILE ===============================
with open(directory+countyDeathFilename, newline='') as csvfile:
    ipdata = csv.reader(csvfile, delimiter=',', quotechar='"')
    deathsDB = []
     # to avoid commas and standardise across DBs. NB OSofT is "deaths_global.csv"
    for row in ipdata:      # bespoke namefixes - if necessary
        if row [stateCol] == "Korea, South": row [countyCol] = 'South Korea'
        if row [countyCol]   == "Bonaire, Sint Eustatius and Saba": row [countyCol] = 'Bonaire Sint Eustatius and Saba'
        # the above lines are for example if name adjustments are necessary
        deathsDB += [[','.join(row)]]      

for indx in range (1,len(deathsDB)):
    deathsDB[indx] = deathsDB[indx][0].split(',')                   # each row of the DB is indexed sequentially as per the original file
#    deathsDB[indx] [LatCol]    = float (deathsDB[indx] [LatCol])    # convert Latitude string to float
#    deathsDB[indx] [LongCol]   = float (deathsDB[indx] [LongCol])   # convert Longditude string to float
    for deathNr in range (dateBeginCol, len (deathsDB[indx])):         # convert the rest (death numbers) to ints
        deathsDB[indx][deathNr] = int (deathsDB[indx][deathNr])

deathsDB[colLabelRow] = deathsDB[colLabelRow][0].split(',')         # The column title texts/\

" ========  pull cases data from file ==========================" 
with open(directory+countyCasesFilename, newline='') as csvfile:
    ipdata = csv.reader(csvfile, delimiter=',', quotechar='"')
    casesDB = []
     # to avoid commas and standardise across DBs. NB OSofT for format, names etc is "deaths_global.csv"
    for row in ipdata:      # bespoke namefixes - if necessary
        if row [stateCol] == "Korea, South": row [countyCol] = 'South Korea'
        if row [countyCol]   == "Bonaire, Sint Eustatius and Saba": row [countyCol] = 'Bonaire Sint Eustatius and Saba'
        # the above lines are for example if name adjustments are necessary
        casesDB += [[','.join(row)]]      

for indx in range (1,len(casesDB)):
    casesDB[indx] = casesDB[indx][0].split(',')                   # each row of the DB is indexed sequentially as per the original file
#    casesDB[indx] [LatCol]    = float (casesDB[indx] [LatCol])    # convert Latitude string to float
#    casesDB[indx] [LongCol]   = float (casesDB[indx] [LongCol])   # convert Longditude string to float
    for caseNr in range (dateBeginCol, len (casesDB[indx])):         # convert the rest (death numbers) to ints
        casesDB[indx][caseNr] = int (casesDB[indx][caseNr])

deathsDB[colLabelRow] = deathsDB[colLabelRow][0].split(',')        # The column title texts/\
casesDB[colLabelRow]  = casesDB[colLabelRow][0].split(',')         # The column title texts/\

  

#           sunday      monday      tuesday   (wed = 1/22/20, the first date in the database, pre pad is used to startCharts on a Sun
#datePad = ['1/19/20','1/20/20', '1/21/20']                          # 1/22/20 (the first date in the DB) is a Wed, so pre pad for a Sunday start
#dataPad = [0, 0, 0]                                                 # insert len ( datePad days of 0 data)
columnLabels = deathsDB[0]
dateSpacer = "."                                                    # a list of the column headings (dates etc)
#print ("============\nColumn Labels\n============\n", columnLabels)

dates = columnLabels[4:]                                  # Sunday-padded dates   #print (dates)
#print (dates)

deathsDB = deathsDB[1:]                                    # drop the column headings
casesDB  = casesDB[1:]                                     # drop the column headings

monthDict= {'01':'Jan ', '02':'Feb ', '03':'Mar ', '04':'Apr ','05':'May ', '06':'June',
            '07':'July', '08':'Aug ', '09':'Sep ', '10':'Oct ','11':'Nov ','12':'Dec ' }

def convertDates(spacer= "" , alphaMonth = False):
    """ converts dates in DB (yyyy-mm-dd) to international (yyyySmmSdd)"""
    for indx in range (len(dates)):
        #print (dates [indx] , end= " ")                     # dates start at col 4
        tmp = dates[indx].split('-')                         # [mm, dd, yy] as per the dataset
        year    = tmp[0]+' '
        month   = ('0' +tmp[1])[-2:]                         # make sure it is 2 digits
        day     = ('0' +tmp[2])[-2:]                         # make sure it is 2 digits
        if alphaMonth:
            month = monthDict[month]
            dates[indx]= year + month + day #[yyyyJan.dd]. S = spacer
        else:
            dates[indx]= year + dateSpacer + month + day #[yyyyJan.dd]. S = spacer


convertDates(".", alphaMonth=True )

#deathsDB = deathsDB[1:]                                             # drop the column headings

#print ("============\nColumn Labels\n============\n", dates)

stateList = []              # collate all state counties into a single state entry
for state in deathsDB:       # make a lit of all states with counties (should be all)
    if state[0] != '':
        if state[1] not in stateList:
            stateList += [state[1]]

for country in stateList:                                                # compute the running sums of contries in states
    runningDeathsSum = ['',country,'','']+ [0] * (len (columnLabels) - 4)    # we just want the death data clugey
    for state in deathsDB:                                                # find each state for that country
        if state [1] == country:
            for indx in range (4, len (runningDeathsSum)):
                runningDeathsSum [indx] = runningDeathsSum [indx] + state [indx]
#    print (runningDeathsSum)
    deathsDB += [runningDeathsSum]                                           #   add new country summation to the deathsDB

# if no state, make state == country
for prov in deathsDB:
    if prov[stateCol]=="":
        prov[stateCol] = prov [countyCol]


stateDeathsDict = {}                 # a dict to hold all country, state and summed-state data
for state in deathsDB:               # Collates all states into a single dictionary entry
    if state[0] == "":
        stateDeathsDict [state[1]]  = state        # there is no state, the dict key is country
    else:
        stateDeathsDict [state[0]]  = state        # there is more than on state so use the dick key is the state

def printAllstates():
    kees = sorted (stateDeathsDict.keys())               # alpha sort for pretty print
    for keeNr in range (len (kees)):
        print (kees[keeNr], end="\t\t")
#        if keeNr % 3 == 0: print()

# =========== PULL THE Country Population Data DB FILE =================================
#directory="/Users/davidworrall/Documents/RESEARCH/COVID-19/dataSets/"
#directory="/volumes/DRWORANGE32/COVID-19/dataSets/"
#with open('/Users/davidworrall/Documents/CCC/TEACHING/TeachingResources_DSon/LiteracyRatesGlobalLongRun.csv', newline='') as csvfile:

colLabelRow     = 0
regionNameCol   = 0
countryNameCol  = 1
countrySymbolCol= 2
areaCol         = 3
populationCol   = 4
densityCol      = 5
densityFactor   = 100000    # used to calculate the density - easily changed

with open(directory+populationDataFilename, newline='') as csvfile:
    ipdata = csv.reader(csvfile, delimiter=',', quotechar='"')
    populationDB = []
    for row in ipdata:                                              # bespoke state namefixes
        if row [countryNameCol] == "Korea, South": row [countryNameCol] = 'South Korea'
        if row [countryNameCol] == "Korea, North": row [countryNameCol] = 'North Korea'
        if row [countryNameCol] == "Bonaire, Sint Eustatius and Saba": row [countryNameCol] = 'Bonaire Sint Eustatius and Saba'
        if row [countryNameCol] == 'Taiwan*': row [countryNameCol] = 'Taiwan'
        if row [countryNameCol] == "Cote d'Ivoire": row [countryNameCol] = 'Cote dIvoire'

        if row [countryNameCol] == 'Congo (Kinshasa)': row [countryNameCol] = 'Congo-Kinshasa (DR)'
        if row [countryNameCol] == 'Congo (Brazzaville)': row [countryNameCol] = 'Congo-Brazzaville'
        if row [countryNameCol] == 'Holy See': row [countryNameCol] = 'Vatican'
        if row [countryNameCol] == 'Sint': row [countryNameCol] = 'Sint Maarten'
        if row [countryNameCol] == 'Pierre': row [countryNameCol] = 'Sant Pierre and Miquelon'
        if row [countryNameCol] == 'Terres': row [countryNameCol] = 'Terres Australes'
        if row [countryNameCol] == 'Smaller': row [countryNameCol] = 'Smaller Territories (BR)'
        if row [countryNameCol] == 'St Martin': row [countryNameCol] = 'Saint Martin'

        populationDB += [[','.join(row)]]
for indx in range (1,len(populationDB)):
    populationDB[indx] = populationDB[indx][0].split(',')                      # each row of the DB is indexed sequentially as per the original file
    try:    populationDB[indx][areaCol] = float (populationDB[indx][areaCol])  # try convert from str
    except: populationDB[indx][areaCol] = 0.0                                  # or set to 0.0
    try:    populationDB[indx][populationCol] = float(populationDB[indx][populationCol]) # try convert from str or set to 0.0
    except: populationDB[indx][populationCol] = 0.0
    if populationDB[indx][populationCol] != 0.0:                               # no area so not recorded
        populationDB[indx][densityCol] = populationDB[indx][areaCol] / populationDB[indx][populationCol]

populationDB[colLabelRow] = populationDB[colLabelRow][0].split(',')            # The column title texts/\
# print (populationDB[colLabelRow])


# check which countries in the populationDB are or are not in the stateDeathsDict.keys())
deathstateNames = sorted (stateDeathsDict.keys())
inBothDB = []
inJustDeathDB = []
inJustPopulationDB = []
for stateNr in range (2, len (populationDB)):
#    print (populationDB[stateNr][countryNameCol])
    if populationDB[stateNr][countryNameCol] in deathstateNames:
        inBothDB += [populationDB[stateNr][countryNameCol]]
    else:
        inJustDeathDB += [populationDB[stateNr][countryNameCol]]



# ====================DEFINE DEATH CLASS FOR Counties and States

class CountryDeaths():    # define a class for handling Covid-19 death data for a country
    def __init__(self, countryName):
        """Initialize data attributes."""
        self.countryName        = countryName
        self.countryExists      = False
        self.printData          = False                 # used to filter death data when using print (instance)
        self.stateName       = ""                    # filled from deahDB  by __init__()
        self.region             = ""                    # filled from populationDB  by __init__()
        self.latitude           = stateList[LatCol]
        self.longitude          = stateList[LatCol]
        self.area               = 0
        self.population         = 0
        self.populationDensity  = 0
                                    #sun     mon       tuea          wed
        self.prePadDates        = ['1/19/20','1/20/20', '1/21/20'] # 1/22/20 = first data date. prepad for a Sunday start
        self.prePadData         = [0] * len (self.prePadDates)     # insert len ( self.prePadDates days of data = 0)
        self.cumulativeDeaths   = self.prePadData                  # start the list by prepadding
        self.deathIncrements    = []  #NB clculaed from self.cumulativeDeaths, so no need to prePad
        self.ascending          = []
        self.sortedIndices      = []                    # indices of increments in ascending order
        self.NrDataPoints       = 0                     #Needed many time so no need to keep calculating it
        self.mean               = 0
        self.max                = 0
        self.simpleMovingAveragePeriod = 0
        self.SMAdata            = []

        if self.countryName not in stateDeathsDict.keys():
            self.countryExists = False
            print (self.countryName + " does not exist in the Database.\nIt contains data for the following countries:")
 #           kees = sorted (stateDeathsDict.keys())    # alpha sort for 'pretty' print
 #           for keeNr in range (len (kees)):
#                print (kees[keeNr], end="\t")
 #               if keeNr % 3 == 0: print()               # throw a CR
            return (None)
        else:
            self.countryExists = True
#            print("*********", stateDeathsDict[self.countryName])
#            print
            self.stateName   = stateDeathsDict[countryName][0] #stateList[self.countryName][0]
            self.latitude       = stateDeathsDict[countryName][2] # stateList[LatCol]
            self.longitude      = stateDeathsDict[countryName][3] # stateList[LongCol]
            self.area           = None
            self.population     = None
            self.cumulativeDeaths = self.prePadData + stateDeathsDict[countryName][4:]
            # block the printing of values <=0
            for indx in range (len(self.cumulativeDeaths)):
                if self.cumulativeDeaths[indx] <= 0:
                    self.cumulativeDeaths[indx] = 0 # float ('nan')     # nan = not-a-number            #
            self.deathIncrements = [0]* len (self.cumulativeDeaths)
            for dayNr in range (1,len (self.cumulativeDeaths)):
                self.deathIncrements[dayNr] = self.cumulativeDeaths[dayNr]-self.cumulativeDeaths[dayNr-1]
                if self.deathIncrements[dayNr] <0:                      # indicates data error
                    if self.printData:
                        print ("DATA ERROR:",self.countryName , end="\t")
#                      print ("DATA ERROR:Some dead rose again in ",self.countryName , "on", :", end="/t")
                        print (dayNr,":",dates[dayNr], "\t", self.cumulativeDeaths[dayNr-1],"\t",self.cumulativeDeaths[dayNr],"\tdiff :", self.deathIncrements[dayNr-1],self.deathIncrements[dayNr])

                    self.deathIncrements[dayNr] = 0  # just zero it : #  ============ ERROR CORRECTION==============
            self.generateStats()
            #  ============== NBNBNBNBNBNBN
            self.zeros2NANs()                       # disable printing of 0's
            #print (self.deathIncrements)
            if self.stateName == "":
                self.stateName = self.countryName # the 'state' is the country
            self.NrDataPoints = len (self.deathIncrements)

#            countriesInConsiderationList += [self.countryName]

    def __str__(self):
        """ Defines what to do when the default python help() function is called on an instance"""
        if self.countryExists == True:
            print ('#' * 30)
            print ('Country:', self.countryName)
            print ('#' * 30)
            print ('state:\t' , self.stateName)
            print ('Latitude:\t' , self.latitude)
            print ('Longitude:\t' , self.longitude)
            print ('Area (sq kms):\t', self.area)
            print ('Population:\t' , self.population)
            print ('Mean of new deaths:\t', self.mean)
            print ('MaxVal of new deaths:\t', self.max )
            if not self.printData:
                print ("Switch self.printData on for more complete details of"+self.countryName)
            if self.printData:
                print (self.countryName+': Cumulative Deaths:\n', self.cumulativeDeaths)
                print (self.countryName+': Death Increments:\n', self.deathIncrements)
                print (self.countryName+': Sorted deathIncrements:\n', self.ascending)
        return ("")

    def generateStats (self):
        # generates some stats of the death
        from statistics import mean
        self.mean       = mean(self.deathIncrements) #; print ('Daily deaths, mean   :', self.mean)
        self.max        = max (self.deathIncrements) #; print ('Daily deaths, maxVal :', self.max )
        self.ascending  = sorted(self.deathIncrements)
#        sorted(range(len(vals)), key=vals.__getitem__)
        self.sortedIndices = sorted(range(len(self.deathIncrements)), key=self.deathIncrements.__getitem__)
#        sorted(range(len(vals)), key=vals.__getitem__)


    def SMA (self, NrPeriods):
        """ Computes a Simple Moving Average of self.deathIncrements over NrPeriods"""
        self.simpleMovingAveragePeriod = NrPeriods
        self.SMAdata = []
        lastDataPointIndex = len (self.deathIncrements)
        if lastDataPointIndex <  self.simpleMovingAveragePeriod:
            print ("Averaging period too long." +
                   "Longest period is: ", lastDataPointIndex-self.simpleMovingAveragePeriod)
            return(None)
        index = 0
        while index < self.NrDataPoints:
            thisWindow = self.deathIncrements[index: index+ self.simpleMovingAveragePeriod]
            windowAverage = sum(thisWindow) / self.simpleMovingAveragePeriod
#            print (thisWindow, windowAverage )
            self.SMAdata += [windowAverage]  # .append (windowAverage)
            index += 1

    def zeros2NANs (self):
        # replace all zeros with float ('nan') [Not A Number] to suppress printing of all 0 values
        nrDays = len(self.deathIncrements)
        for indx in range (1, nrDays): # don't use the first one
            if self.deathIncrements[indx] > (200 + self.deathIncrements[indx-1]):  # attempt at filtering outliers
                pass
            # print (self.deathIncrements[indx-1], self.deathIncrements[indx])
            #self.deathIncrements[indx] = 0 # self.deathIncrements[indx-1]
        for indx in range (nrDays):  # leave last value to set manually
            if self.deathIncrements[indx] <= 0:       # zero -ve increments:resurrection currently not available :-)
               self.deathIncrements[indx] = float('nan')
        self.deathIncrements[0]=0               # Kludge: force a printable val so graph will start at the 0th index
        if not (self.deathIncrements [-1] > 0):
            self.deathIncrements += [0]         # Kludge: force a printable val so graph will end  at the 0th index

    #=================================
import matplotlib.pyplot as plt
# =================================
covidInstanceDict = {}   # used to  store country instances for plotting etc
def graphCountries(listOfCountryNames,graphName, nrFigureCols = 1, directToFile=False):
    " draws a graph of a list of country instances. If directToFile=True, closes the plot display after saving an image file"
    title = 'Daily Covid-19 Reported Deaths: '+graphName+'\n'+ dates[0][4:]+ " " + dates[0][:4] + '—' +dates[-1][4:]+ " " + dates[-1][:4]
    plt.figure (num = 1, figsize = (18.0,12.0))
    plt.title (title, fontsize=14)
    plt.ylabel("Number of Confirmed Deaths Reported", fontsize=9)
    plt.xlabel("Daily Dates (Sundays labeled)", fontsize=9)
    plt.grid(axis='both', which='both', color='grey', linestyle='-', linewidth=0.15)  # was 0.2
    plt.xticks (rotation = 45) #'vertical'
    xlabels = dates #datePad + columnLabels[4:]  # the dates. The first is Wednesday, Jan 22. Need to add 3 days to the beginning
    xmarkers = list (range(0,len(xlabels)+7, 7)) # one per week
    xlabels2 = [xlabels[datey][-6:] for datey in range(0,len(xlabels), 7)]  # omot year- take just month and day
    plt.xticks(xmarkers, xlabels2, fontsize=10)

    for country in listOfCountryNames:
          instanceString = "covidInstanceDict['"+country+"'] = CountryDeaths('"+country+"')"
#          print ("==================")
          exec(instanceString)                  # create the instance
#          print("attempt to execute NANs:", instanceString+'.zeros2NANs()')

          dgStr1 = "plt.plot(covidInstanceDict['"
          dgStr2 = "'].deathIncrements, label='"
          dgStr3 ="', marker='.', linewidth='0.35')"
          dataGraphString = dgStr1 +country+ dgStr2 +country+ dgStr3
#          print ('datagraphString:', dataGraphString)
          exec(dataGraphString)                         #  store a  created country instance in the country dictionaty

    plt.legend(title='Country/Region', loc = 'upper left', ncol=nrFigureCols, shadow = False)  # wish list + transparent=True
    datey = dates[-1].replace(dateSpacer,"").replace(" ","")    # take out the separator for the filename
    plt.savefig(imageDirectory+graphName+'_covid19_'+datey+'.png', format = 'png',
                orientation='landscape', transparent=False )
    plt.show()
    if directToFile: #  close the Figure #  default is false -- SWITCH TO TRUE TO eact
#        pass
        plt.close()

"""
              # EasternEurope,'Eastern Europe'
                                 # Americas, 'The Americas'
AmericaCentral1     = ['Mexico', 'Brazil', 'Guatemala', 'Bolivia',]

AmericaCentral2      = ['Belize', 'Cuba', 'Honduras',  'Panama', 'Nicaragua']
AmericaNorth        = ['Canada', 'Alberta','British Columbia', 'Diamond Princess', 'Grand Princess', 'Manitoba',
                       'New Brunswick', 'Newfoundland and Labrador', 'Northwest Territories', 'Nova Scotia',
                       'Ontario', 'Prince Edward Island', 'Quebec', 'Saskatchewan', 'US']
AmericaSouth        = ['Venezuela', 'Peru', 'Argentina', 'Uruguay', 'Falkland Islands (Malvinas)', 'French Guiana',
                       'Guyana','Paraguay']

AfricaNorthern1     = ['Senegal' ,'Mauritania', 'Western Sahara', 'Mali', 'Niger', 
                      'Libya','Chad',  'Eritrea','Somalia','Djibouti', 'South Sudan', 'Suriname']
AfricaNorthern2     = ['Egypt', 'Algeria', 'Morocco','Sudan',]
#AfricaCentral       = ['Ghana', 'Burkina Faso', 'Nigeria', 'Ga'zbon', 'Equatorial Guinea', 'Congo',
#                       'Zambia', 'Tanzania', 'Malawi', "Cote dIvoire", 'Togo', 'Benin', 'Burkina Faso']
AfricaCentral1       = ['Ghana',  'Zambia', 'Nigeria','Malawi',]
AfricaCentral2       = ['Burkina Faso', 'Tanzania',  'Togo', 'Benin', 'Burkina Faso', 'Cote dIvoire']  # 'Cote dIvoire', 
AfricaWest          = ['Guinea', 'Guinea-Bissau','Gambia', 'Sierra Leone', 'Liberia',  'Sao Tome and Principe']
AfricaEastern       = ['Mayotte', 'Mauritius',  'Mozambique', 'Rwanda',  'Seychelles', 'Uganda'] #'Eastern Africa',
AfricaSouthern1      = ['Namibia', 'Botswana', 'Angola','Zimbabwe','Madagascar', 'Kenya', 'Lesotho' ]
AfricaSouthern2      = ['South Africa' ]

Asia                = ['China', 'Anhui', 'Beijing', 'Chongqing', 'Fujian', 'Gansu', 'Guangdong', 'Guangxi',
                       'Guizhou', 'Hainan', 'Hebei', 'Heilongjiang', 'Henan', 'Hong Kong', 'Hubei',
                       'Hunan', 'Inner Mongolia', 'Mongolia', 'Jiangsu', 'Jiangxi', 'Jilin', 'Liaoning', 'Macau',
                       'Ningxia', 'Qinghai', 'Shaanxi', 'Shandong', 'Shanghai', 'Shanxi', 'Sichuan',
                       'Tianjin', 'Tibet', 'Xinjiang', 'Yunnan', 'Zhejiang', 'Turks and Caicos Islands',
                       'South Korea', 'Japan','Taiwan', 'Nepal', 'Burma', 'Bhutan', 'Maldives']  # NB no North Korea
AsiaNoChina         = [ 'Turks and Caicos Islands','South Korea', 'Japan','Taiwan', 'Nepal', 'Burma', 'Bhutan', 'Maldives']  # NB no North Korea
AsiaSouth           = ['Bangladesh', 'India','Pakistan', 'Sri Lanka' ]  #  'Myanmar', 
AsiaSouth1           = ['Laos', 'Thailand', 'Vietnam', 'Cambodia', 'Mongolia', 'Tajikistan']
AsiaSouth2           = ['Pakistan', 'India', 'Bangladesh', 'Sri Lanka', 'Laos', 'Thailand', 'Vietnam', 'Cambodia', 'Mongolia', 'Tajikistan' ] #'Myanmar', 
AsiaSouth3           = ['India',]
AsiaSouthEast       = ['Philippines','Malaysia','Singapore','Indonesia', 'Papua New Guinea','Timor-Leste']

Canada1              = ['Alberta', 'Northwest Territories','British Columbia',  'Manitoba', 'New Brunswick', 'Newfoundland and Labrador',
                        'Nova Scotia', 'Prince Edward Island', 'Saskatchewan'] #plus 'Canada' 'Diamond Princess', 'Grand Princess',
Canada2              = ['Ontario', 'Quebec' ] #plus 'Canada' 'Diamond Princess', 'Grand Princess',

Carribean1           = ['Anguilla', 'Antigua and Barbuda', 'Aruba', 'Bahamas', 'Barbados', 'Bermuda',
                        'Cayman Islands', 'Grenada', 'Guadeloupe', 'Haiti',]

Carribean2           = ['Jamaica', 'Martinique', 'Saint Kitts and Nevis', 'Saint Lucia', 'Saint Pierre and Miquelon',
                        'Saint Vincent and the Grenadines', 'Sint Maarten', 'Trinidad and Tobago', 'Tunisia']
Carribean3           = ['Anguilla', 'Antigua and Barbuda', 'Aruba', 'Bahamas', 'Barbados', 'Bermuda',
                        'Cayman Islands', 'Grenada', 'Guadeloupe', 'Haiti', 'Jamaica', 'Martinique', 'Saint Kitts and Nevis', 'Saint Lucia', 'Saint Pierre and Miquelon',
                        'Saint Vincent and the Grenadines', 'Sint Maarten', 'Trinidad and Tobago', 'Tunisia']

Chinesestates1    = ['Anhui', 'Beijing', 'Chongqing', 'Fujian', 'Gansu', 'Guangdong', 'Guangxi', 'Guizhou']
ChineseProvinces2    = ['Hainan', 'Hebei', 'Heilongjiang', 'Henan', 'Hong Kong', 'Hubei', 'Hunan', 'Inner Mongolia', 'Mongolia']
ChineseProvinces3    = ['Jiangsu', 'Jiangxi', 'Jilin', 'Liaoning', 'Macau', 'Ningxia', 'Qinghai', 'Shaanxi', ]
ChineseProvinces4    = ['Shandong', 'Shanghai', 'Shanxi', 'Sichuan', 'Tianjin', 'Tibet', 'Xinjiang', 'Yunnan', 'Zhejiang']

EuropeNorthEastern  = ['Belarus', 'Estonia', 'Iceland','Kosovo', 'Latvia', 'Lithuania', 'Poland' ,
                        'Russia', 'Kyrgyzstan', 'Slovenia']
EuropeCentralEastern1 = ['Austria', 'Bosnia and Herzegovina', 'Czechia','Hungary', 'Moldova','Romania',
                        'Serbia', 'Slovakia', 'Ukraine' ]
#EuropeCentralEastern1 = ['Austria', 'Bosnia and Herzegovina', 'Czechia','Hungary', 'Moldova','Romania',
#                        'Serbia', 'Slovakia', 'Ukraine' ]
EuropeNorthWestern1  = [ 'Denmark', 'Finland', 'Iceland', 'Norway', 'Sweden']
EuropeNorthWestern2  = [ 'Denmark', 'Finland', 'Iceland', 'Norway']
EuropeNorthWestern3  = [ 'Sweden']

EuropeSouthEastern  = ['Albania', 'Armenia', 'Azerbaijan', 'Georgia']
EuropeSouthWestern1  = ['Andorra', 'Bulgaria',  'Greece', 'Gibraltar', 'Ireland', 'Isle of Man',
                        'Liechtenstein', 'Luxembourg','Malta', 'Monaco', 'Montenegro',  'Netherlands',
                        'North Macedonia', 'Portugal', 'San Marino',  'Switzerland',  ]
EuropeSouthWestern2  = ['Andorra',  'Greece', 'Gibraltar', 'Ireland', 'Isle of Man',
                        'Liechtenstein', 'Luxembourg','Malta', 'Monaco', 'Montenegro',  'Netherlands',
                        'North Macedonia', 'Portugal', 'San Marino',  'Switzerland',  ]
EuropeSouthWestern3 = [ 'Belgium', 'France','Germany', 'Italy', 'Spain', 'United Kingdom' ]

MiddleEast1         = ['Bahrain', 'Jordan',  'Lebanon', 'Oman',
                       'Qatar', 'Syria', 'Tajikistan', 'United Arab Emirates', 'Uzbekistan', 'West Bank and Gaza',
                       'Yemen']
MiddleEast2         = ['Iran', 'Iraq', 'Saudi Arabia', 'Turkey','Israel', 'Kuwait','Kazakhstan', 'Oman',]

Oceania             = ['French Polynesia', 'New Caledonia', 'Australia','Australian Capital Territory', 'New South Wales', 'Northern Territory',
                       'Queensland', 'South Australia', 'Tasmania', 'Victoria', 'Western Australia',
                       'New Zealand']
AmericaUSA             = ['US']

AustralianStates       = ['Australian Capital Territory', 'New South Wales', 'Northern Territory',
                       'Queensland', 'South Australia', 'Tasmania', 'Victoria', 'Western Australia']
Australasia        =  ['New Zealand', 'Australian Capital Territory', 'New South Wales', 'Northern Territory',
                       'Queensland', 'South Australia', 'Tasmania', 'Victoria', 'Western Australia']

noVictoria        =  ['New Zealand', 'Australian Capital Territory', 'New South Wales', 'Northern Territory',
                       'Queensland', 'South Australia', 'Tasmania', 'Western Australia']


 
#Figures now render in the Plots pane by default. 
#To make them also appear inline in the Console, uncheck 
#"Mute Inline Plotting" under the Plots pane options menu. 
"""
"""
graphCountries (AfricaNorthern1,        'Northern Africa 1of2', nrFigureCols = 3, directToFile=True)

graphCountries (AfricaNorthern2,        'Northern Africa 2of2',          directToFile=True)
graphCountries (AfricaCentral1,          'Central Africa 1of2',           directToFile=True)
graphCountries (AfricaCentral2,          'Central Africa 2of2',           directToFile=True)
graphCountries (AfricaWest,             'West Africa',              directToFile=True)
graphCountries (AfricaEastern,          'Eastern Africa',           directToFile=True)
graphCountries (AfricaSouthern1,         'Southern Africa (no South Africa)',          directToFile=True)
graphCountries (AfricaSouthern2,         'Southern Africa: South Africa',          directToFile=True)

graphCountries (AmericaCentral1,        'Central America 1of2 (Brazil & Mexico)', directToFile=True)
graphCountries (AmericaCentral2,        'Central America 2of2',        directToFile=True)

graphCountries (AmericaUSA,                    'United States',            directToFile=True)

graphCountries (AsiaNoChina,            'Asia-no China',       directToFile=True)
graphCountries (AsiaSouth1,             'South Asia 1of2',               directToFile=True)
graphCountries (AsiaSouth2,             'South Asia 2of2',               directToFile=True)
graphCountries (AsiaSouth3,             'South Asia 2of2',               directToFile=True)
graphCountries (AsiaSouthEast,          'South-East Asia',           directToFile=True)

graphCountries (Australasia,            'Australasian Territories', directToFile=True)
graphCountries (Canada1,                'Canadian Provinces-no Ontario or Quebec', directToFile=True)
graphCountries (Canada2,                'Canada: Ontario & Quebec', directToFile=True)

graphCountries (ChineseProvinces1,       'Chinese Provinces 1of4',   directToFile=True)
graphCountries (ChineseProvinces2,       'Chinese Provinces 2of4',        directToFile=True)
graphCountries (ChineseProvinces3,       'Chinese Provinces 3of4',        directToFile=True)
graphCountries (ChineseProvinces4,       'Chinese Provinces 4of4',        directToFile=True)

graphCountries (Carribean1,              'Carribean 1of3',        nrFigureCols = 2, directToFile=True)
graphCountries (Carribean2,              'Carribean 2of3',        nrFigureCols = 2, directToFile=True)
graphCountries (Carribean3,              'Carribean 3of3 (All)',  nrFigureCols = 2, directToFile=True)

#graphCountries (EuropeNorthWestern1,     'North-Western Europe',     directToFile=True)
graphCountries (EuropeNorthWestern2,     'North-Western Europe (no Sweden)',     directToFile=True)
graphCountries (EuropeNorthWestern3,     'North-Western Europe: Sweden)',     directToFile=True)
graphCountries (EuropeNorthEastern,     'North-Eastern Europe',     directToFile=True)
graphCountries (EuropeCentralEastern1,   'Central-Eastern Europe 1of 1',   directToFile=True)

graphCountries (EuropeSouthWestern1,     'South-Western Europe 1of2',     nrFigureCols = 1, directToFile=True)
graphCountries (EuropeSouthWestern2,     'South-Western Europe 2of2', directToFile=True)
graphCountries (EuropeSouthWestern3,     'South-Western Europe 2of2', directToFile=True)
graphCountries (EuropeSouthEastern,     'South-Eastern Europe',     directToFile=True)

graphCountries (MiddleEast1,            'Middle East (1of2)',        directToFile=True)
graphCountries (MiddleEast2,            'Middle East (2of2)',        directToFile=True)
#graphCountries (Asia,                   'Asia',                     directToFile=True)
graphCountries (Oceania,                'Oceania',                  directToFile=True)

#graphCountries (NorthAmerica, 'North America', directToFile=True)
#graphCountries (AustralianStates,       'Australian States',        directToFile=True)
#graphCountries (noVictoria,             'Australia (no Victoria)',  directToFile=True)

#"""


"""
allSoFar=NorthWesternEurope +NorthEasternEurope +CentralEasternEurope +SouthWesternEurope
allSoFar2 =SouthEasternEurope +SouthAsia +NorthernAfrica +CentralAfrica +SouthernAfrica
allSoFar3 = Asia +SouthAsia +SouthEastAsia + MiddleEast
allSoFar += allSoFar2 +allSoFar3
allSoFar.sort()
print (len (allSoFar))
# identity check
#for province in provinceDeathsDict.keys():
#    if province not in allSoFar :
#        print (province)

#directory="/volumes/DRWORANGE32/COVID-19/dataSets/"
#aussie = CountryDeaths('Australia')#
#graphCountries (Australasia , 'Aussie')
#graphCountries(NorthWesternEurope, 'North-Western Europe')
notYetClassified = []
#  ============== Get Extract Regional Data ===================================
fname= "WorldRegionsUpdated.csv"

with open(directory+fname, newline='') as csvfile:
    ipdata = csv.reader(csvfile, delimiter=',', quotechar='"')
    for row in ipdata:
#        print (row[0])
        if row [0] in notYetClassified:
            print (row[:2])  # get the region

# "us-counties20200516.csv"






from datetime import datetime

for datey in range (DateBeginCol, len(deathsDB[colLabelRow])):
    deathsDB[colLabelRow][datey] = datetime.strptime(deathsDB[colLabelRow][datey], "%m/%d/%y").date()
#    print (deathsDB[colLabelRow][datey])

for datey in range (DateBeginCol, len(deathsDB[colLabelRow])-7, 4):
    print (deathsDB[colLabelRow][datey+7]- deathsDB[colLabelRow][datey])
    print (deathsDB[colLabelRow][datey])


dateDict={}


countyList = []

dateList = []
for place in deathsDB:
    if place[0] not in dateList:
        dateList += [place[0]]
dateList = dateList[1:]                                     # drop the column title
#print (dateList)
#print ("==========dates\n", dateList)
"""
"""
plt.title("Daily Covid-19 Deaths\nfor the period "+columnLabels[4]+ '–' +columnLabels[-1], fontsize=18)
plt.ylabel("Number of Confirmed Deaths", fontsize=12)
plt.legend(title='Country/Region')
plt.grid(axis='both', which='both', color='grey', linestyle='-', linewidth=0.25)
"""
    # tests of class CountryDeaths()
"""

aust = CountryDeaths("Australia")       #; countriesInConsiderationList += ['aust']
nz=CountryDeaths("New Zealand")
iceland = CountryDeaths("Iceland")
china = CountryDeaths("China")
plt.plot(china.deathIncrements)
plt.plot(china.SMAdata)
china.SMA(3)
plt.plot(china.SMAdata)
brazil =    CountryDeaths('Brazil') ; plt.plot(brazil.deathIncrements,    label='Brazil') # ; brazil.SMA(3) ; plt.plot(brazil.SMAdata)
china =     CountryDeaths('China') ; plt.plot(brazil.deathIncrements,    label='China') # ; brazil.SMA(3) ; plt.plot(brazil.SMAdata)
france =    CountryDeaths('France')  ; plt.plot(france.deathIncrements,     label='France')
india =     CountryDeaths('India')  ; plt.plot(india.deathIncrements,     label='India')
mexico =    CountryDeaths('Mexico') ; plt.plot(mexico.deathIncrements,    label='Mexico')
#pakistan =  CountryDeaths('Pakistan'); plt.plot(pakistan.deathIncrements,  label='Pakistan')
india =     CountryDeaths('India')  ; plt.plot(india.deathIncrements,     label='India')
russia =    CountryDeaths('Russia") ; plt.plot(russia.deathIncrements,    label='Russia')
spain =     CountryDeaths('Spain") ; plt.plot(spain.deathIncrements,    label='Spain")
#sweden =    CountryDeaths("Sweden") ; plt.plot(sweden.deathIncrements,    label='Sweden')
UK =        CountryDeaths('United Kingdom") ; plt.plot(UK.deathIncrements,  label='United Kingdom')
USA =       CountryDeaths('USA')     ; plt.plot(USA.deathIncrements,       label='United States')
"""
